#
# spaceship class
# spaceship.py
#

import math
import random
import pygame
from pygame.locals import *
import enemy
import gameobj
import vector

SPRITE = None
TIME_TILL_DROP = 6.0
TIME_TILL_LAND = 2.0
TIME_TILL_LEAVE = 2.0

def load_spaceship():
	global SPRITE
	SPRITE = pygame.image.load("spaceship.png").convert_alpha()

class spaceship(gameobj.gameobj):
	def __init__(self,world,dest_pos,drop_info):
		global SPRITE
		gameobj.gameobj.__init__(self,world)
		self._level_collision = False
		self._state = "fly-in"
		self._drop_info = drop_info
		self._dest_pos = dest_pos
		self._sprite = SPRITE
		angle = random.randint(0,359)
		self._pos = self._world._objs[self._world._player_id]._pos + vector.twin(
			math.cos(math.radians(angle)),math.sin(math.radians(angle))
			) * self._world._res[0] * 1.2
		self._orig_pos = self._pos
		self._heading = dest_pos - self._pos
		self._dist = self._heading.len()
		self._orig_dist = self._dist
		self._heading.normalize()
		self._speed = self._dist / TIME_TILL_DROP # we want it to arrive in constant time
		self.size = 128
		self._draw_offset = vector.twin(self.size / 2.0, self.size / 2.0)
	
	def draw(self,pos):
		self._world._screen.blit(self._sprite,(pos - self._draw_offset).get())
	
	def set_id(self,id):
		self._id = id
		
	def process(self,time_passed):
		gameobj.gameobj.process(self,time_passed)
		# normal movement
		self._dist -= (time_passed * self._speed) / 1000.0
		if self._dist <= 0:
			if self._state == "fly-in":
				self._state = "dropping"
				self._pos = self._dest_pos # correcting location to avoid landing into walls
				self._heading = vector.twin(0,0)
				self._dist = TIME_TILL_LAND * self._speed
			elif self._state == "dropping":
				self._state = "fly-out"
				self._heading = self._orig_pos - self._pos
				self._dist = self._heading.len()
				self._heading.normalize()
				self._heading = self._heading * -1
				self._speed = self._dist / TIME_TILL_LEAVE
				id = self._world.add_obj(enemy.enemy(self._world,self._drop_info))
				self._world._objs[id]._pos = self._pos
			elif self._state == "fly-out":
				self._world.del_obj(self._id)
