#
# world class object
# world.py
#

import pdb
import pygame
from pygame.locals import *
import hud
import enemy
import player
import shot
import level
import spaceship
import vector

LOST = 0
WON = 1

class world:
	def __init__(self,screen,resolution,levelname,score):
		self._debug_flag = False
		# load textures
		level.load_tiles()
		shot.load_shots()
		spaceship.load_spaceship()
		# generate world
		self._res = resolution
		self._screen = screen
		self._score = score
		self._objs = {}
		self._obj_id = 0
		self._player_id = 0
		self._background = pygame.Surface(self._res,0,24).convert()
		grass = pygame.image.load("grass.png").convert()
		for x in range(0,self._res[0],grass.get_width()):
			for y in range(0,self._res[1],grass.get_height()):
				self._background.blit(grass,(x,y))
		# level
		self._level = level.level(self,levelname)
		self.populate()
		# hud
		self._hud = hud.hud(self,self._player_id)
	
	def process(self,time_passed):
		# player controls
		pkeys = pygame.key.get_pressed()
		player_mx = 0
		player_my = 0
		if pkeys[K_w]:
			player_my -= 1
		if pkeys[K_s]:
			player_my += 1
		if pkeys[K_a]:
			player_mx -= 1
		if pkeys[K_d]:
			player_mx += 1
		pv = vector.twin(player_mx,player_my)
		pv.normalize()
		self._objs[self._player_id]._heading = pv	
		# processing level
		if not self._level.process(time_passed):
			return (WON,self._score)
		# moving objects
		for i in self._objs.keys():
			if not self._objs.has_key(i):
				# object was killed
				continue
			self._objs[i].process(time_passed)
			if not self._objs.has_key(i):
				# object was killed
				continue
			# check level collision
			if self._objs[i]._level_collision:
				ctype = level.TYPE_PASS
				if self._objs[i].__class__.__name__ == shot.shot.__name__:
					ctype = level.TYPE_SHOOT
				if self._level.collide(self._objs[i]._pos,self._objs[i].size,ctype):
					self._objs[i].collided(time_passed)
				if not self._objs.has_key(i):
					# object was killed
					continue
				# check shots collision with players
				if self._objs[i].__class__.__name__ == shot.shot.__name__:
					for j in self._objs.keys():
						if self._objs[j].__class__.__name__ in [player.player.__name__,enemy.enemy.__name__]:
							if self._objs[j].check_hit(self._objs[i]):
								self.del_obj(i)
								if self._objs[j]._life <= 0:
									if j == self._player_id:
										return (LOST,self._score)
									else:
										self._level.enemy_died(self._objs[j])
										self.del_obj(j)
								break
		return None
	
	def set_player(self,id):
		self._player_id = id
	
	def add_obj(self,obj):
		self._obj_id += 1
		self._objs[self._obj_id] = obj
		return self._obj_id
		
	def del_obj(self,objid):
		del self._objs[objid]
	
	def draw(self):
		# for stuff relative to player
		diff = vector.twin(self._res[0] / 2.0,self._res[1] / 2.0) - self._objs[self._player_id]._pos
		
		# draw background
		self._screen.fill((0,0,0))
		self._level.draw(diff)
		
		# drawing objects
		for i in self._objs:
			obj = self._objs[i]
			obj.draw(obj.get_pos() + diff)
		# drawing hud
		self._hud.draw()
	
	def populate(self):
		# level must be loaded
		# player
		po = player.player(self)
		po._pos = vector.twin(self._level._data["playerpos"])
		poid = self.add_obj(po)
		self.set_player(poid)
		# enemies
	
	def process_mousedown(self,event):
		if event.button == 1:
			stype = shot.SIMPLE
			if pygame.key.get_mods() & KMOD_SHIFT:
				stype = shot.POWER
			self._objs[self._player_id].shoot(stype,(vector.twin(event.pos) - vector.twin(self._res) / 2.0))
		if event.button == 3:
			stype = shot.GRENADE
			self._objs[self._player_id].shoot(stype,(vector.twin(event.pos) - vector.twin(self._res) / 2.0))

		